# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# fmt: off
from ......schema_classes import ChartSnapshotClass
from ......schema_classes import CorpGroupSnapshotClass
from ......schema_classes import CorpUserSnapshotClass
from ......schema_classes import DashboardSnapshotClass
from ......schema_classes import DataFlowSnapshotClass
from ......schema_classes import DataJobSnapshotClass
from ......schema_classes import DataPlatformSnapshotClass
from ......schema_classes import DataProcessSnapshotClass
from ......schema_classes import DatasetSnapshotClass
from ......schema_classes import GlossaryNodeSnapshotClass
from ......schema_classes import GlossaryTermSnapshotClass
from ......schema_classes import MLFeatureSnapshotClass
from ......schema_classes import MLFeatureTableSnapshotClass
from ......schema_classes import MLModelSnapshotClass
from ......schema_classes import MLPrimaryKeySnapshotClass
from ......schema_classes import TagSnapshotClass


ChartSnapshot = ChartSnapshotClass
CorpGroupSnapshot = CorpGroupSnapshotClass
CorpUserSnapshot = CorpUserSnapshotClass
DashboardSnapshot = DashboardSnapshotClass
DataFlowSnapshot = DataFlowSnapshotClass
DataJobSnapshot = DataJobSnapshotClass
DataPlatformSnapshot = DataPlatformSnapshotClass
DataProcessSnapshot = DataProcessSnapshotClass
DatasetSnapshot = DatasetSnapshotClass
GlossaryNodeSnapshot = GlossaryNodeSnapshotClass
GlossaryTermSnapshot = GlossaryTermSnapshotClass
MLFeatureSnapshot = MLFeatureSnapshotClass
MLFeatureTableSnapshot = MLFeatureTableSnapshotClass
MLModelSnapshot = MLModelSnapshotClass
MLPrimaryKeySnapshot = MLPrimaryKeySnapshotClass
TagSnapshot = TagSnapshotClass
# fmt: on
