# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# fmt: off
from .....schema_classes import DatasetDeprecationClass
from .....schema_classes import DatasetFieldMappingClass
from .....schema_classes import DatasetLineageTypeClass
from .....schema_classes import DatasetPropertiesClass
from .....schema_classes import DatasetUpstreamLineageClass
from .....schema_classes import EditableDatasetPropertiesClass
from .....schema_classes import UpstreamClass
from .....schema_classes import UpstreamLineageClass


DatasetDeprecation = DatasetDeprecationClass
DatasetFieldMapping = DatasetFieldMappingClass
DatasetLineageType = DatasetLineageTypeClass
DatasetProperties = DatasetPropertiesClass
DatasetUpstreamLineage = DatasetUpstreamLineageClass
EditableDatasetProperties = EditableDatasetPropertiesClass
Upstream = UpstreamClass
UpstreamLineage = UpstreamLineageClass
# fmt: on
