# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# fmt: off
from ......schema_classes import BaseDataClass
from ......schema_classes import CaveatDetailsClass
from ......schema_classes import CaveatsAndRecommendationsClass
from ......schema_classes import EthicalConsiderationsClass
from ......schema_classes import EvaluationDataClass
from ......schema_classes import IntendedUseClass
from ......schema_classes import IntendedUserTypeClass
from ......schema_classes import MLFeaturePropertiesClass
from ......schema_classes import MLFeatureTablePropertiesClass
from ......schema_classes import MLModelFactorPromptsClass
from ......schema_classes import MLModelFactorsClass
from ......schema_classes import MLModelPropertiesClass
from ......schema_classes import MLPrimaryKeyPropertiesClass
from ......schema_classes import MetricsClass
from ......schema_classes import QuantitativeAnalysesClass
from ......schema_classes import SourceCodeClass
from ......schema_classes import SourceCodeUrlClass
from ......schema_classes import SourceCodeUrlTypeClass
from ......schema_classes import TrainingDataClass


BaseData = BaseDataClass
CaveatDetails = CaveatDetailsClass
CaveatsAndRecommendations = CaveatsAndRecommendationsClass
EthicalConsiderations = EthicalConsiderationsClass
EvaluationData = EvaluationDataClass
IntendedUse = IntendedUseClass
IntendedUserType = IntendedUserTypeClass
MLFeatureProperties = MLFeaturePropertiesClass
MLFeatureTableProperties = MLFeatureTablePropertiesClass
MLModelFactorPrompts = MLModelFactorPromptsClass
MLModelFactors = MLModelFactorsClass
MLModelProperties = MLModelPropertiesClass
MLPrimaryKeyProperties = MLPrimaryKeyPropertiesClass
Metrics = MetricsClass
QuantitativeAnalyses = QuantitativeAnalysesClass
SourceCode = SourceCodeClass
SourceCodeUrl = SourceCodeUrlClass
SourceCodeUrlType = SourceCodeUrlTypeClass
TrainingData = TrainingDataClass
# fmt: on
