# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# fmt: off
from ......schema_classes import TransformationTypeClass
from ......schema_classes import UDFTransformerClass


TransformationType = TransformationTypeClass
UDFTransformer = UDFTransformerClass
# fmt: on
